Option Explicit

'-----------------------------------------------------------------------------------------
' Procedure : fncExtractText
' Created   : Jan 3 2005 11:40 (yep, first verstion created way back then!!)
' Reference : fncExtractText*
' Author    : Michael Reese
' Technical : This procedure may not be optimized for fastest performance. It is constructed
'           :   to provide a clear understanding of the underlying principles of text extraction.
' Inputs    : (strTextBlock, required) Text block that contains text to extract
'           : (strLeft, optional) Identified text at the immediate left of text to extract
'           : (strRight, optional) Identified text at the immediate right of text to extract
'           : (lngStart, optional) Starting point to search for strLeft
'           : (blnIncludeLeft, optional) Includes identified text left of, along with extracted text
'           : (blnIncludeRight, optional) Includes identified text right of, along with extracted text
'           : (strFlag, optional) Text that identifies the lngStart point for extracting text
'           : (blnDeleteText, optional) Deletes, rather than extracts, text from text block
'           : (lngReturnError, optional) Indicates error return message options
'           : (strErrorText, optional) Text returned when error is detected
'           : (lngCompare, optional) Indicates whether characters returned are case sensitive
' Output    : Extracted text, or text block with extracted text deleted
' Purpose   : To extract one element of text (text) from a text block, or delete one element
'           :   from the text block
' Misc.     : If lngStart is passed as -1, then strFlag indicates the last
'           :   occurence of text within the text block to assign value of lngStart
'           : If lngStart is passed as negative long number less than -1, then strFlag
'           :   indicates the nth occurrence of text ("n" is calculated by taking the absolute
'           :   value of lngStart) within the text block to assign value of lngStart
'-----------------------------------------------------------------------------------------
Public Function fncExtractText( _
      ByVal strTextBlock As String, _
      Optional ByVal strLeft As String, _
      Optional ByVal strRight As String, _
      Optional ByVal lngStart As Long = 1, _
      Optional ByVal blnIncludeLeft As Boolean, _
      Optional ByVal blnIncludeRight As Boolean, _
      Optional ByVal strFlag As String, _
      Optional ByVal blnDeleteText As Boolean, _
      Optional ByVal lngReturnError As Long, _
      Optional ByVal strErrorText As String, _
      Optional ByVal lngCompare As Long = 1 _
      ) As String

'|<------ 90-character width -------------------------------- 90-character width ------->|

PROC_DECLARATIONS:
   Dim strExtractText    As String
   Dim strOrigTextBlock  As String
   Dim strDeleteText     As String
   Dim strExtender       As String
   Dim strError          As String
   Dim lngError          As Long
   Dim lngLeftStart      As Long
   Dim lngIndex          As Long
   Dim blnExtender       As Boolean

PROC_START:
   On Error GoTo PROC_ERROR
   strExtender = ""
   strOrigTextBlock = strTextBlock
   lngError = vbFalse

PROC_MAIN:
   'Establish the starting point within the main text from which to locate the text to extract
   If lngStart = -1 Then
      'Assign starting point at the last occurrence of strFlag in Text block
      lngStart = InStr(InStrRev(strTextBlock, strFlag), strTextBlock, strLeft, lngCompare)
   ElseIf lngStart < -1 Then
      'Assign starting point at the nth occurrence of strFlag in Text block
      lngLeftStart = Abs(lngStart)
      lngStart = 1
      For lngIndex = 1 To lngLeftStart
         lngStart = InStr(lngStart + 1, strTextBlock, strFlag, lngCompare)
         If lngStart = 0 Then
            'There are less occurrences than the one desired; use last occurrence instead
            lngStart = InStr(InStrRev(strTextBlock, strFlag), strTextBlock, strLeft, lngCompare)
            Exit For
         End If
      Next
   ElseIf Len(strFlag) > 0 Then
      'Assign starting point at the first occurence of strFlag in Text block
      lngStart = InStr(lngStart, strTextBlock, strFlag, lngCompare) + (Len(strFlag) - 1)
   ElseIf lngStart = 0 Or _
         Len(strLeft) = 0 Then
      'If lngStart is erroneously passed as 0, or if a string extender is needed at
      'beginning of text to be extracted, assign starting point to a value of 1
      lngStart = 1
   End If

   'Special error assignments which can assist the user in case of common errors
   If lngStart = 1 Then
      If Len(strLeft) = 0 Then
         If InStr(1, strTextBlock, strRight, lngCompare) = 0 Then
            'strRight not located in text
            lngError = 1
         End If
      Else
         If Len(strRight) > 0 Then
            If InStr(lngStart, strTextBlock, strLeft, lngCompare) > 0 Then
               If InStr(lngStart, strTextBlock, strLeft, lngCompare) > _
                     InStr(InStr(lngStart, strTextBlock, strLeft, lngCompare), _
                     strTextBlock, strRight, lngCompare) Then
                  'strRight is not located after strLeft in text after lngStart
                  lngError = 3
               End If
            Else
               'strLeft not located in text
               lngError = 2
            End If
         End If
      End If

   Else
      If Len(strRight) > 0 Then
         If InStr(lngStart, strTextBlock, strLeft, lngCompare) > 0 Then
            lngLeftStart = InStr(lngStart, strTextBlock, strLeft, lngCompare)
            If InStr(lngStart, strTextBlock, strLeft, lngCompare) > InStr(lngLeftStart, strTextBlock, strRight, lngCompare) Then
               'strRight is not located after strLeft in text after lngStart
               lngError = 3
            End If
         Else
            'strLeft not located in text
            lngError = 2
         End If
      End If
   End If

   If lngError <> vbFalse Then
      GoTo PROC_ERROR
   End If

   If Len(strLeft) = 0 Or _
         Len(strRight) = 0 Then

      'in the unlikely event that strExtender is present in current text block, keep
      'adding random obscure ASCII characters until it doesn't match anything
      Do While InStr(strTextBlock, strExtender) > 0
         strExtender = strExtender & Chr$(Int((255 - 21 + 1) * Rnd + 21))
      Loop

      'strExtender is used to provide the reference text to the beginning (left) or end
      '(right) of the strings, if no designated left or right text indicators are included
      If strLeft = Space(0) Then
         strTextBlock = strExtender & strTextBlock
         strLeft = strExtender
         blnIncludeLeft = False
         blnExtender = True
      End If

      If strRight = Space(0) Then
         strTextBlock = strTextBlock & strExtender
         strRight = strExtender
         blnIncludeRight = False
         blnExtender = True
      End If

   End If

   'TEXT EXTRACTION PROCESS
   'The text extraction process can be done in one of two ways: Through the multi-line
   'extraction process, or through the single line extraction process. Read each expla-
   'nation paragraph. NOTE: Each process will produce identical results.

   '--------------------------------------------------------------------------------------
   'MULTI-LINE EXTRACTION: This section comprises of the multi-line sequence in extracting
   'the text from the text block. This process shows a step-by-step process into how the
   'extracted text is identified, and extracted from the main text block. If you wish to
   'monitor the process more effectively, then uncomment the four dimension statements,
   'uncomment the following five statements, and comment the single-line extraction code
   'in the following section.

   'Dim lngLeft           As Long
   'Dim lngRight          As Long
   'Dim lngLeftAdjust     As Long
   'Dim lngExtractText    As Long

   'lngLeft = InStr(lngStart, strTextBlock, strLeft, lngCompare)
   'lngRight = InStr(lngLeft + 1, strTextBlock, strRight, lngCompare)
   'lngLeftAdjust = lngLeft + Len(strLeft)
   'lngExtractText = lngRight - lngLeftAdjust
   'strExtractText = Mid(strTextBlock, lngLeftAdjust, lngExtractText)
   '--------------------------------------------------------------------------------------

   '--------------------------------------------------------------------------------------
   'SINGLE-LINE EXTRACTION: This section creates the functionality of the multi-line
   'extraction process into a single line of code, with line extensions included.
   'You can just use this stripped single-line code in your own procedure if you
   'have the following variables, providing they are pre-verified for no error:
   'strTextBlock, strLeft, strRight, lngStart, and lngCompare. If you wish to use this
   'method, then comment over the four dimension statements and five statements in the
   'multi-line extraction method.

   strExtractText = Mid(strTextBlock, (InStr(lngStart, strTextBlock, strLeft, lngCompare)) + _
         Len(strLeft), (InStr((InStr(lngStart, strTextBlock, strLeft, lngCompare)) + 1, _
         strTextBlock, strRight, lngCompare)) - ((InStr(lngStart, strTextBlock, strLeft, _
         lngCompare)) + Len(strLeft)))
   '--------------------------------------------------------------------------------------

   'Re-appends left text tag to beginning of extracted text
   If blnIncludeLeft Then
      strExtractText = strLeft & strExtractText
   End If

   'Re-appends right text tag to end of extracted text
   If blnIncludeRight Then
      strExtractText = strExtractText & strRight
   End If

   'Remove all strExtender references
   strExtractText = Replace(strExtractText, strExtender, Space(0))

   'If text were designated for deletion rather than extraction, remove
   'from original text and return the result (inverse of extraction)
   If blnDeleteText Then
      strDeleteText = strExtractText
      strExtractText = Replace(strOrigTextBlock, strDeleteText, Space(0))
   End If

PROC_EXIT:
   fncExtractText = strExtractText
   Exit Function

PROC_ERROR:
   If lngError > vbFalse Then
      Select Case lngReturnError
         Case 0
            'error message activated and empty string returned
            strError = Choose(lngError, _
                  "No parameter text indicated for strRight variable """ & strRight & """ located in body text segment", _
                  "No parameter text indicated for strLeft variable """ & strLeft & """ located in body text segment", _
                  "Parameter text for strLeft variable """ & strLeft & """ does not precede " & vbCrLf & _
                        "parameter text for strRight variable """ & strRight & """ in body text segment")
            strExtractText = Space(0)
            Call MsgBox(strError, vbOKOnly + vbExclamation)
         Case 1
            'error message suppressed; if strErrorText passed then
            'strErrorText custom text is returned; otherwise empty string is returned
            strExtractText = strErrorText
         Case 2
            'error message suppressed; entire original string returned
            strExtractText = strTextBlock
         Case 3
            'normal VB error message box display, as in "else" condition
            MsgBox "Error " & Err.Number & " (" & _
                    Err.Description & ")" & vbCrLf & vbCrLf & _
                    "Procedure: fncExtractText" & vbCrLf & _
                    "Module: (none indicated)"
      End Select

   Else
      MsgBox "Error " & Err.Number & " (" & _
              Err.Description & ")" & vbCrLf & vbCrLf & _
              "Procedure: fncExtractText" & vbCrLf & _
              "Module: (none indicated)"
   End If

   GoTo PROC_EXIT

End Function
